module namespace ttgen="Transformation Tool Code Generator";

import module namespace ttl="XMI Read Library" at "../TTXMILib/readLib.xq";

declare namespace xmi="http://schema.omg.org/spec/XMI/2.1";
declare namespace uml="http://schema.omg.org/spec/UML/2.0";
declare namespace fn="http://www.w3.org/2005/xpath-functions";
declare namespace xs="http://www.w3.org/2001/XMLSchema";


declare function ttgen:xmiToXsdml($doc as node()) as node()*{
	<fileSet>
	{
		for $x in ttl:getClasses($doc)
		return(
			ttgen:spreadFileCode($x, ttl:getOwnedMemberByType($doc, "uml:Association"), ttl:getOwnedMemberByType($doc, "uml:DataType"), ttl:getClasses($doc), ttl:getPackages($doc))
		)
	}
	</fileSet>
};

declare function ttgen:spreadFileCode($x as node(), $assoc as node ()*, $types as node()*, $classes as node()*, $packages as node()*) as node(){
	document{
		<file id="{data(concat($x/@xmi:id,"TTClassFile"))}">
			{
				ttgen:spreadPackage($x/@xmi:id, $packages)
			}
			
			{
				ttgen:spreadSwingImport()
			}
			
			{
				ttgen:spreadClass($x, $assoc, $types, $classes)
			}
		</file>
	}
};

declare function ttgen:spreadPackage($id as xs:string, $packages as node ()*) as node(){
	if(fn:empty(ttl:getPackageByClassID($id, $packages)[1])) then (
		document{data("")}
	)else(
 			let $classPack := ttl:getPackageByClassID($id, $packages)
			 return(
			<package id="{data(ttl:getPackageByClassID($id, $packages)[1]/@xmi:id)}" qname="{data(ttl:getPackageByClassID($id, $packages)[1]/@name)}">
				<kw>package</kw>
				<sp>{data(" ")}</sp>
				<qName id="{data(concat(ttl:getPackageByClassID($id, $packages)[1]/@xmi:id,"TTPackageQName"))}">
					<ident>{data(ttl:getPackageByClassID($id, $packages)[1]/@name)}
						{
							let $first := ttl:getPackageByClassID($id, $packages)[1]							
							for $x in ttl:getPackageByClassID($id, $packages)
							where $x/@xmi:id != $first/@xmi:id
							return(
								concat(".", $x/@name)
							)
						}				
					</ident>
				</qName>
				<op>;</op>
			</package>
			)
	)
};

declare function ttgen:spreadSwingImport() as node(){
	document{
			<import id="swingTTImport" qname="javax.swing.*">
				<kw>import</kw>
				<sp>{data(" ")}</sp>
				<qName id="swingTTImportQName">
					<ident>javax</ident><op>.</op><ident>swing</ident><op>.</op><op>{data("*")}</op>
				</qName>
				<op>{data(";")}</op>
			</import>
	}
};

declare function ttgen:spreadClass($x as node(), $assoc as node()*, $types as node()*, $classes as node()*) as node(){
	<class access="{data($x/@visibility)}" id="{data(concat($x/@xmi:id,"TTClass"))}">
		<kw>{data($x/@visibility)}</kw>
		<sp>{data(" ")}</sp>
		<kw>class</kw>
		<sp>{data(" ")}</sp>
		<ident defid="{data(concat($x/@xmi:id,"TTClass"))}">{data($x/@name)}</ident><sp>{data(" ")}</sp>
		
		{
			ttgen:spreadGeneralization($x, $classes)
		}
		
		<op>{data("{")}</op>				
				
		{
			ttgen:spreadClassAttributes($x, $assoc, $types, $classes)
		}
				
		{
			ttgen:spreadClassMethods($x, $types, $classes)
		}
												
		<op>{data("}")}</op>
	</class>
};

declare function ttgen:spreadGeneralization($x as node(), $classes as node()*) as node()*{
	let $gen := ttl:getClassGeneralizations($x)
	return (		
		if(fn:empty($gen)) then (
		)else(
			let $gen1 := $gen[1]
			return(
				<extdOpt id="{data(concat($gen1/@xmi:id,"TTExtdOpt"))}">
					<sp>{data(" ")}</sp>
					<kw>extends</kw>
					<sp>{data(" ")}</sp>
					<type fqn="{data(ttgen:getClassNameByID($gen1/@general, $classes))}" id="{data(concat($gen1/@general,"TTType"))}" sort="Object">
						<ident>{data(ttgen:getClassNameByID($gen1/@general, $classes))}</ident>
					</type>
					<sp>{data(" ")}</sp>
				</extdOpt>
			)
		)		
	)
};

declare function ttgen:spreadClassAttributes($x as node(), $assoc as node()*, $types as node()*, $classes as node()*) as node()*{					
	(
		for $y in ttl:getClassAttributes($x)
		return(
			ttgen:spreadAttribute($y, $types, $classes)
		)
	)
	
	union
	
	(
		for $y in ttl:getClassCompositions($x, $assoc)
		return(
				ttgen:spreadCompositeAttribute(ttl:getCompositionSource($y), ttl:getCompositionDest($y), $classes)
		)
	)
};

declare function ttgen:spreadClassMethods($x as node(), $types as node()*, $classes as node()*) as node()*{
	(: Constructors :)					
					
	(
		for $y in ttl:getClassConstructors($x)
		return(
			ttgen:spreadConstructor($y, $types, $classes)
		)
	)
					
	union
					
	(: Methods left :)
	(
		for $y in ttl:getClassNonConstructorMethods($x)
		return(
			ttgen:spreadMethod($y, $types, $classes)
		)
	)
};

declare function ttgen:spreadAttribute($x as node(), $types as node()*, $classes as node()*) as node(){
	document{
		<field access="{data(ttgen:getAttributeVisibility($x))}" id="{data(concat($x/@xmi:id,"TTAttribute"))}">
			<kw>{data(ttgen:getAttributeVisibility($x))}</kw>
			<sp>{data(" ")}</sp>
			
			{
				ttgen:spreadType($x, $types, $classes)
			}
			
			<sp>{data(" ")}</sp>
			<ident defid="{data(concat($x/@xmi:id,"TTIdent"))}">{data($x/@name)}</ident>
						
			{
				if($x/xmi:Extension/typeModifier/@xmi:value = "[]") then( 
					ttgen:spreadTypeModifier($x/xmi:Extension, ttgen:spreadType($x, $types, $classes))
				)else()
			}
			
			<op>{data(";")}</op>
		</field>
	}
};

declare function ttgen:getAttributeVisibility($x as node()) as xs:string{
	if($x/@visibility = "") then(
		"public"
	)else(
		$x/@visibility
	)
};

declare function ttgen:spreadCompositeAttribute($sour as node(), $dest as node(), $classes) as node(){
	document{
		<field access="private" id="{data(concat($sour/@association,"TTField"))}">
			<kw>private</kw>
			<sp>{data(" ")}</sp>
			
			{
				<type fqn="{data(ttgen:getClassNameByID($sour/@type, $classes))}" 
							id="{data(concat($sour/@type,"TTType"))}" sort="Object">
					<ident>{data(ttgen:getClassNameByID($sour/@type, $classes))}</ident>
				</type>
			}
			
			<sp>{data(" ")}</sp>
			<ident defid="{data(concat($sour/@association,"TTIdent"))}">{data($sour/@name)}</ident>
						
			{
				if((fn:compare($sour/lowerValue/@value, "*") = 0) or (fn:compare($sour/upperValue/@value, "*") = 0)) then(
					<type fqn="{data(concat(ttgen:getClassNameByID($dest/@type, $classes),"[]"))}" 
								id="{data(concat($dest/@type,"TTModifier"))}" sort="Array">
						<op>[]</op>
					</type>
				)else()
			}
			
			<op>{data(";")}</op>
		</field>
	}
};

declare function ttgen:spreadConstructor($x as node(), $types as node()*, $classes as node()*) as node(){
	document{
			<ctor access="{data($x/@visibility)}" id="{data(concat($x/@xmi:id,"TTCtor"))}">
			<kw>{data($x/@visibility)}</kw>
			<sp>{data(" ")}</sp>
			<ident defid="{data(concat($x/@xmi:id,"TTCtor"))}">{data($x/@name)}</ident>
			<op>{data("(")}</op>
			
			{
				ttgen:spreadMethodInOutParameters($x, $types, $classes)
			}
						
			<op>{data(")")}</op>
			<op>{data("{")}</op>
			<op>{data("}")}</op>
		</ctor>
	}
};

declare function ttgen:spreadMethod($x as node(), $types as node()*, $classes as node()*) as node(){
	document{
		<method access="{data($x/@visibility)}" id="{data(concat($x/@xmi:id,"TTMethod"))}">
			<kw>{data($x/@visibility)}</kw>
			<sp>{data(" ")}</sp>
			
			{
				ttgen:spreadType(ttl:getMethodReturnParameter($x), $types, $classes)
				union
				(
					if(ttl:getMethodReturnParameter($x)/xmiExtension/typeModifier/@xmi:value = "[]") then( 
						ttgen:spreadTypeModifier(ttl:getMethodReturnParameter($x)/xmi:Extension, ttgen:spreadType(ttl:getMethodReturnParameter($x), $types, $classes)))
					else()
				)
			}
			
			<sp>{data(" ")}</sp>
			<ident defid="{data(concat($x/@xmi:id,"TTCtor"))}">{data($x/@name)}</ident>
			<op>{data("(")}</op>		
			
			{
				ttgen:spreadMethodInOutParameters($x, $types, $classes)
			}
			
			<op>{data(")")}</op>
			<op>{data("{")}</op>
			<op>{data("}")}</op>
		</method>
	}
};

declare function ttgen:spreadMethodInOutParameters($x as node(), $types as node()*, $classes as node()*) as node()*{
	(
		let $num := fn:count(ttl:getMethodInOutParameters($x))-1
		return(
			if($num > 0) then (
				for $i in (1 to $num)
				return(
					(ttgen:spreadMethodInOutParameter(ttl:getMethodInOutParameters($x)[$i], $types, $classes))
					union					
					(<op>,</op>)
				)
			)else()
		)
	)
	
	union
	
	(
		let $num := fn:count(ttl:getMethodInOutParameters($x))
		return (
			if($num > 0) then (
				ttgen:spreadMethodInOutParameter((ttl:getMethodInOutParameters($x)[$num]), $types, $classes)
			)else()
		)
	)
};

declare function ttgen:spreadMethodInOutParameter($x as node(), $types as node()*, $classes as node()*) as node(){
	document{
		<param id="{data(concat($x/@xmi:id,"TTParam"))}">
			{
				ttgen:spreadType($x, $types, $classes)
			}
			<sp>{data(" ")}</sp>
			<ident defid="{data(concat($x/@xmi:id,"TTIdent"))}">{data($x/@name)}</ident>
			{
				if($x/xmi:Extension/typeModifier/@xmi:value = "[]") then( 
					ttgen:spreadTypeModifier($x/xmi:Extension, ttgen:spreadType($x, $types, $classes)) 
				)else()
			}
		</param>
	}
};

declare function ttgen:spreadType($param as node(), $types as node()*, $classes as node()*) as node(){
	(: primitive type :)
	if(ttgen:isPrimitiveType($param, $types)[1]) then (
		<type fqn="{data(ttgen:getTypeNameByID($param/@type, $types))}" id="{data(concat($param/@type,"TTType"))}" sort="Primitive">
			<ident>{data(ttgen:getTypeNameByID($param/@type, $types))}</ident>
		</type>
	)else(
		(: class type :)
		if(ttgen:isClassType($param, $classes)[1]) then (
			<type fqn="{data(ttgen:getTypeNameByID($param/@type, $classes))}" id="{data(concat($param/@type,"TTType"))}" sort="Object">
				<ident>{data(ttgen:getTypeNameByID($param/@type, $classes))}</ident>
			</type>
		)else(
			(: omitted void type :)
			<type fqn="void" id="void_idTTType" sort="Primitive"><ident>void</ident></type>
		)			
	)
};

declare function ttgen:spreadTypeModifier($extension as node(), $origType as node()) as node(){
	<type fqn="{data(concat($origType/@fqn,"[]"))}" id="{data(concat($origType/@id,"TTModifier"))}" sort="Array">
		<op>[]</op>
	</type>
};

declare function ttgen:isPrimitiveType($param as node(), $types as node()*) as xs:boolean*{
	for $x in $types
	where $param/@type = $x/@xmi:id
	return true()
};

declare function ttgen:isClassType($param as node(), $classes as node()*) as xs:boolean*{
	for $x in $classes
	where $param/@type = $x/@xmi:id
	return true()
};

declare function ttgen:getTypeNameByID($id as xs:string, $types as node()*) as xs:string*{
	for $x in $types
	where $id = $x/@xmi:id
	return $x/@name
};

declare function ttgen:getClassNameByID($id as xs:string, $classes as node()*) as xs:string*{
	for $x in $classes
	where $id = $x/@xmi:id
	return $x/@name
};
