module namespace ttsgen="Transformation Tool Smart Code Generator";

import module namespace ttl="XMI Read Library" at "../TTXMILib/readLib.xq";
import module namespace ttgen="Transformation Tool Code Generator" at "./plainTransform.xq";
import module namespace ttjava="Java classes identified as graphic components" at "./javaLib.xq";

declare namespace xmi="http://schema.omg.org/spec/XMI/2.1";
declare namespace uml="http://schema.omg.org/spec/UML/2.0";
declare namespace fn="http://www.w3.org/2005/xpath-functions";
declare namespace xs="http://www.w3.org/2001/XMLSchema";

declare function ttsgen:smartXmiToXsdml($doc as node()) as node(){
	<fileSet>
	{
		for $x in ttsgen:getNonJavaClassFiles($doc)
		return ttsgen:spreadFileCode($x, ttsgen:getNonJavaClassFiles($doc)//class)
	}
	</fileSet>
};

declare function ttsgen:getNonJavaClassFiles($doc as node()) as node()*{
	for $x in ttgen:xmiToXsdml($doc)//file
	where ttjava:isGraphicComponent($x//class/ident/text()) != fn:true()
	return $x
};

declare function ttsgen:spreadFileCode($file as node(), $classes as node()*) as node(){
	document{
		<file id="{data($file/@id)}">
			{
				$file/package
			}
			{
				ttsgen:spreadJavaImports($file/class)
			}
			{
				ttsgen:spreadClass($file/class, $classes)
			}
		</file>
	}
};

declare function ttsgen:spreadJavaImports($class as node()) as node()*{
	(ttsgen:spreadJavaAwtImports($class))
	union
	(for $x in ttsgen:getJavaClassReferences($class)
	return ttsgen:spreadJavaSwingImport($x))
};

declare function ttsgen:spreadJavaAwtImports($class as node()) as node()*{
	document{
		<import id="awtTTImport" qname="java.awt.FlowLayout">
            <kw>import</kw>
            <sp>{data(" ")}</sp>
            <qName id="awtTTImportQName">
                <ident>java</ident>
                <op>.</op>
                <ident>awt</ident>
                <op>.</op>
                <op>FlowLayout</op>
            </qName>
            <op>;</op>
    </import>
    
    union
    
		(
			if(ttsgen:classContainsActionEnabledComponent($class))then(
				(
				<import id="awtActionEventTTImport" qname="java.awt.event.ActionEvent">
            <kw>import</kw>
            <sp>{data(" ")}</sp>
            <qName id="awtActionEventTTImportQName">
                <ident>java</ident>
                <op>.</op>
                <ident>awt</ident>
                <op>.</op>
                <op>event</op>
                <op>.</op>
                <op>ActionEvent</op>                
            </qName>
            <op>;</op>
    		</import>
		    )
		    
		    union
		    
		    (
		    <import id="awtActionListenerTTImport" qname="java.awt.event.ActionListener">
            <kw>import</kw>
            <sp>{data(" ")}</sp>
            <qName id="awtActionListenerTTImportQName">
                <ident>java</ident>
                <op>.</op>
                <ident>awt</ident>
                <op>.</op>
                <op>event</op>
                <op>.</op>
                <op>ActionListener</op>                
            </qName>
            <op>;</op>
    		</import>
		    )
			)else(
			)
		)
	}
};

declare function ttsgen:spreadJavaSwingImport($name as xs:string) as node()*{
document{
		<import id="swingTTImport" qname="javax.swing.{data($name)}">
            <kw>import</kw>
            <sp>{data(" ")}</sp>
            <qName id="swingTTImportQName">
                <ident>javax</ident>
                <op>.</op>
                <ident>swing</ident>
                <op>.</op>
                <op>{data($name)}</op>
            </qName>
            <op>;</op>
     </import>        				
	}	
};

declare function ttsgen:spreadClass($class as node(), $classes as node()*) as node()*{
	if(ttsgen:isInheritanceByClassName($ttjava:containers, $class/ident/text(), $classes) = fn:true()) then(
		<class access="{data($class/@access)}" id="{data($class/@id)}">
			<kw>{data($class/@access)}</kw>
    	<sp>{data(" ")}</sp>
			<kw>class</kw>
			<sp>{data(" ")}</sp>
			{$class/ident}
			<sp>{data(" ")}</sp>
			{$class/extdOpt}
			<op>{data("{")}</op>
			{$class/field}
			{ttsgen:spreadClassMethods($class, $classes)}
			<op>{data("}")}</op>
		</class>
	)else(
		$class
	)						
};

declare function ttsgen:isInheritanceByClassName($inhnames as xs:string*, $classname as xs:string, $classes as node()*) as xs:boolean{
	if(fn:empty(index-of($inhnames, $classname))) then (
		let $class := ttsgen:getClassByName($classname, $classes)
		return(
			if(fn:empty($class)) then(
				fn:false()
			)else(
				if(fn:empty($class/extdOpt) = fn:false())then(
					ttsgen:isInheritanceByClassName($inhnames, $class/extdOpt/type/ident/text(), $classes)
				)else(
					fn:false()
				)
			)
		)
	)else(
		fn:true()
	)	
};

declare function ttsgen:getJavaClassReferences($class as node()) as xs:string*{
	distinct-values(
		for $x in $class//ident
		where ttjava:isGraphicComponent($x/text()) = fn:true()
		return $x/text()
	)
};

declare function ttsgen:getClassByName($name as xs:string, $classes as node()*) as node()*{
	if(fn:empty($name)) then()
	else (
			let $class := for $x in $classes where fn:compare($x/ident/text(), $name) = 0 return $x
			return(
			if(fn:empty($class) = fn:false()) then ($class[1])
			else ()
		)
	)
};

declare function ttsgen:spreadClassMethods($class as node(), $classes as node()*) as node()*{
document{
		(: constructor :)
		ttsgen:spreadConstructor($class, $classes)

		union
		
		(: getters :)
		ttsgen:spreadGetters($class, $classes)	
		
		union
		
		(:the rest:)
		ttsgen:spreadOtherMethods($class, $classes)
	}	
	
};

declare function ttsgen:spreadConstructor($class as node(), $classes as node()*) as node(){
	let $ctor := $class/ctor
	return(
		if(fn:empty($ctor)) then(
			<ctor access="public" id="{data($class/ident/text())}">
				<kw>public</kw>
				<sp>{data(" ")}</sp>
				<ident defid="{data($class/ident/text())}">{data($class/ident/text())}</ident>
				<op>{data("(")}</op>
				<op>{data(")")}</op>
				<op>{data("{")}</op>				
				<expr id="setlayoutTTCalling" sort="MethodCall">
				<ident defif="TTsetlayout" fqn="void" ref="java.swing.*">setLayout</ident>
				<op>{data("(")}</op>
				<expr id="newflowlayoutTTCalling" sort="CtorCall">
					<kw>new</kw>
					<sp>{data(" ")}</sp>
					<expr id="TTnewflowlayout" sort="class">
						<ident defid="FlowlayoutClass">FlowLayout</ident>
					</expr>					
					<op>{data("(")}</op>
					<op>{data(")")}</op>
				</expr>
				<op>{data(")")}</op>
			</expr>			
			<op>{data(";")}</op>
			
			{
					for $x in ttsgen:getGraphicAttributes($class, $classes)
					return (
							ttsgen:spreadAddCallByAttribute($x)
							union
							(<op>{data(";")}</op>)
					)
			}
			
			<op>{data("}")}</op>
		</ctor>
		)else(
				<ctor access="public" id="{data($ctor/@id)}">
				<kw>{data($ctor/@access)}</kw>
				<sp>{data(" ")}</sp>
				<ident defid="{data($ctor/ident/@defid)}">{data($ctor/ident/text())}</ident>
				<op>{data("(")}</op>
				
				{
					let $params := $ctor/param
					for $i in (1 to fn:count($params)) 
					return(
						if($i = fn:count($params))then(
							$params[$i]
						)else(
							<op>,</op>
							union
							<sp>{data(" ")}</sp>
							union
							$params[$i]						
						)
					)
				}
							
				<op>{data(")")}</op>
				<op>{data("{")}</op>				
				
				<expr id="setlayoutTTCalling" sort="MethodCall">
					<ident defif="TTsetlayout" fqn="void" ref="java.swing.*">setLayout</ident>
					<op>{data("(")}</op>
					<expr id="newflowlayoutTTCalling" sort="CtorCall">
						<kw>new</kw>
						<sp>{data(" ")}</sp>
						<expr id="TTnewflowlayout" sort="class">
							<ident defid="FlowlayoutClass">FlowLayout</ident>
						</expr>					
						<op>{data("(")}</op>
						<op>{data(")")}</op>
					</expr>
					<op>{data(")")}</op>
				</expr>			
				<op>{data(";")}</op>
				
				{
					for $x in ttsgen:getGraphicAttributes($class, $classes)
					return (
							ttsgen:spreadAddCallByAttribute($x)
							union
							(<op>{data(";")}</op>)
					)
				}
				
				<op>{data("}")}</op>
			</ctor>
		)
	)
};

declare function ttsgen:spreadAddCallByAttribute($x as node()) as node(){
	document{
		<expr id="addTTCalling" sort="MethodCall">
			<ident defif="TTadd" fqn="void" ref="java.swing.*">add</ident>
			<op>{data("(")}</op>		
			<expr id="getterTTCalling" sort="MethodCall">
				<ident defif="TTgetter" fqn="{data($x/type/@fqn)}" ref="java.swing.*">{data(ttsgen:getFieldGetterName($x))}</ident>
				<op>{data("(")}</op>		
				<op>{data(")")}</op>								
			</expr>
			<op>{data(")")}</op>
		</expr>}
};

declare function ttsgen:spreadGetters($class as node(), $classes as node()*) as node()*{
	for $x in ttsgen:getGraphicAttributes($class, $classes)
	return (
		ttsgen:spreadGraphicGetter($x, $classes)
	)
};

declare function ttsgen:spreadGraphicGetter($field as node(), $classes as node()*) as node(){
	document{
		<method access="{data($field/@access)}" id="{data(concat($field/@id, "TTMethod"))}">
			<kw>{data($field/@access)}</kw>
			<sp>{data(" ")}</sp>
			<type fqn="{data($field/type/@fqn)}" id="{data(concat($field/type/@fqn, "_idTTType"))}" sort="Object">
				<ident>{data($field/type/@fqn)}</ident>
			</type>
			<sp>{data(" ")}</sp>
			<ident defid="{data(concat($field/@id, "TTMethod"))}">{data(ttsgen:getFieldGetterName($field))}</ident>
			<op>{data("(")}</op>
			<op>{data(")")}</op>
			<op>{data("{")}</op>
				<stmt id="{data(concat($field/@id, "TTIfStmt"))}" sort="IFELSE">
					<kw>if</kw>
					<op>{data("(")}</op>
					<expr id="{data($field/@id)}" read="yes" sort="VarRef">
						<ident>{data($field/ident/text())}</ident>
					</expr>
					<sp>{data(" ")}</sp>
					<op>{data("==")}</op>
					<sp>{data(" ")}</sp>
					<expr id="s830473259" sort="Literal">
						<literal sort="NULL">null</literal>
					</expr>
					<op>{data(")")}</op>
					<op>{data("{")}</op>
					
					<expr id="{data(concat($field/@id, "TTInstanceCreation"))}" sort="InstanceCreation">
						<expr id="{data(concat($field/@id, "TTObject"))}" read="no" sort="VarRef">
							<ident>{data($field/ident/text())}</ident>							
						</expr>
						<sp>{data(" ")}</sp>
						<op>{data("=")}</op>
						<sp>{data(" ")}</sp>
						<kw>new</kw>
						<sp>{data(" ")}</sp>
						<type fqn="{data($field/type/ident/text())}" id="data{$field/type/@id}" sort="Object">
							<ident>{data($field/type/ident/text())}</ident>
						</type>
						<op>{data("(")}</op>
						<op>{data(")")}</op>						
					</expr>
					<op>{data(";")}</op>
					
					{
						if(ttjava:isTextEnabled($field/type/@fqn)) then(
							<expr id="{data(concat($field/@id, "TTSetText"))}" sort="DOT">
								<expr id="{data(concat($field/@id, "TTSetTextVarRef"))}" read="yes" sort="VarRef">
									<ident>{data($field/ident/text())}</ident>		
								</expr>
								<op>{data(".")}</op>
								<expr id="{data(concat($field/@id, "TTSetTextMethodCall"))}" read="yes" sort="MethodCall">
									<ident>setText</ident>
									<op>{data("(")}</op>
									<expr id="{data(concat($field/@id, "TTSetTextLiteral"))}" sort="Literal">
										<op>"</op>
										<ident>{data($field/ident/text())}</ident>
										<op>"</op>		
									</expr>
									<op>{data(")")}</op>									
								</expr>
							</expr>
							
							union
							
							<op>{data(";")}</op>
						)else(
					
						)
					}
					
					{
						if(ttjava:isActionPerformedEnabled($field/type/@fqn)) then(
							<expr id="{data(concat($field/@id, "TTActionPerformed"))}" sort="DOT">
								<expr id="{data(concat($field/@id, "TTActionPerformedVarRef"))}" read="yes" sort="VarRef">
									<ident>{data($field/ident/text())}</ident>		
								</expr>
								<op>{data(".")}</op>
								<expr id="{data(concat($field/@id, "TTaddActionListenerMethodCall"))}" read="yes" sort="MethodCall">
									<ident>addActionListener</ident>
									<op>{data("(")}</op>

									<expr id="{data(concat($field/@id, "TTActionListenerInstanceCreation"))}" sort="InstanceCreation">
										<kw>new</kw>
										<sp>{data(" ")}</sp>
										<type fqn="ActionListener" id="ActionListener" sort="Object">
											<ident>ActionListener</ident>
										</type>
										<op>{data("(")}</op>
										<op>{data(")")}</op>
										<sp>{data(" ")}</sp>										
										<op>{data("{")}</op>			
										
										<method access="public" id="{data(concat($field/@id, "TTActionPerformedMethod"))}">
											<kw>public</kw>	
											<sp>{data(" ")}</sp>
											<type fqn="void" id="s813695063" sort="Primitive"><kw>void</kw></type>
											<sp>{data(" ")}</sp>
											<ident defid="{data(concat($field/@id, "TTActionPerformedMethodName"))}">actionPerformed</ident>
											<op>{data("(")}</op>
											<param id="{data(concat($field/@id, "TTActionPerformedEventParam"))}">
												<type fqn="java.awt.event.ActionEvent" id="ActionEvent" sort="Object">
													<ident>ActionEvent</ident>
												</type>
												<sp>{data(" ")}</sp>
												<ident defid="{data(concat($field/@id, "TTActionPerformedEventParamObject"))}">e</ident>
											</param>
											<op>{data(")")}</op>
											<op>{data("{")}</op>
											<sp>{data(" ")}</sp>
											<op>{data("}")}</op>
										</method>
										
										<op>{data("}")}</op>					
									</expr>
									
									<op>{data(")")}</op>									
								</expr>
							</expr>
							
							union
							
							<op>{data(";")}</op>							
						)else(
						)
					}
					
					<op>{data("}")}</op>
				</stmt>
				
				<stmt id="{data(concat($field/@id, "TTReturnStmt"))}" sort="RETURN">
					<kw>return</kw>
					<sp>{data(" ")}</sp>	
					<expr id="{data(concat($field/@id, "TTObject"))}" read="no" sort="VarRef">
						<ident>{data($field/ident/text())}</ident>							
					</expr>					
				</stmt>
				<op>{data(";")}</op>																	

			<op>{data("}")}</op>
		</method>
	}
};

declare function ttsgen:spreadOtherMethods($class, $classes) as node()*{
	let $getters := (
		for $x in ttsgen:getGraphicAttributes($class, $classes)
		return fn:lower-case(ttsgen:getFieldGetterName($x))
	)
	for $y in $class/method
	where (fn:empty(index-of($getters, fn:lower-case($y/ident/text())))) = fn:true()
	return $y
};

declare function ttsgen:getGraphicAttributes($class as node(), $classes as node()*) as node()*{
	for $x in $class/field
	where (ttsgen:isInheritanceByClassName($ttjava:containees, $x/type/ident/text(), $classes) = fn:true())
	return $x
};

declare function ttsgen:getFieldGetterName($field as node()) as xs:string{
	concat("get", $field/ident/text())
};

declare function ttsgen:classContainsActionEnabledComponent($class as node()) as xs:boolean{
	if(
		fn:empty(for $x in ttsgen:getJavaClassReferences($class)
		where ttjava:isActionPerformedEnabled($x)
		return $x)
	)then(
		fn:false()
	)else(
		fn:true()
	)
};