module namespace utc="UCD to CD Transformation";

declare namespace xmi="http://schema.omg.org/spec/XMI/2.1";
declare namespace uml="http://schema.omg.org/spec/UML/2.0";
declare namespace fn="http://www.w3.org/2005/xpath-functions";
declare namespace xs="http://www.w3.org/2001/XMLSchema";

import module namespace rl="XMI Read Library" at "../TTXMILib/readLib.xq";
import module namespace wl="XMI Write Library" at "../TTXMILib/writeLib.xq";

declare function utc:namespace($content as node()*, $document as node()) as node()*{
    document{
			<xmi:XMI xmi:version="2.1" xmlns:uml="http://schema.omg.org/spec/UML/2.0" xmlns:xmi="http://schema.omg.org/spec/XMI/2.1">
			<xmi:Documentation exporter="Visual Paradigm for UML" exporterVersion="7.0.1">
				<xmi:Extension extender="Visual Paradigm for UML">
					<nickname exportedFromDifferentName="false"/>
				</xmi:Extension>
			</xmi:Documentation>
            <uml:Model name="{data(rl:getModelName($document))}" xmi:id="{data(rl:getModelID($document))}">
            {$content}
            </uml:Model>
            </xmi:XMI>  
    }
};

declare function utc:ucdTOcd($document as node(), $attributeAsComposition as xs:boolean) as node()*{
	utc:actorTOclass($document, $attributeAsComposition)
	union
	wl:useCasesStereotypes()
	union
	utc:useCaseTOclass($document, $attributeAsComposition)
	union
	utc:includesTOassociation($document)
	union
	utc:associationUCDTOassociationCD($document)
	union
	rl:removeOwnedMemberRepeatedByName(utc:stereotypesTOtypes($document)union utc:taggedValueDefinitionsTOtypes($document))
	union 
	wl:dataType("void", "public", "void_id")
	union(
		if($attributeAsComposition)then(utc:attributesTOassociation($document))
		else()
	)
};

declare function utc:ucdANDactdTOcd($document as node(), $attributeAsComposition as xs:boolean) as node()*{
	utc:actorTOclass($document, $attributeAsComposition)
	union
	wl:useCasesStereotypes()
	union
	utc:useCaseANDactivitiesTOclass($document, $attributeAsComposition)
	union
	utc:includesTOassociation($document)
	union
	utc:associationUCDTOassociationCD($document)
	union
	utc:activitiesTOassociation($document)
	union
	utc:edgesTOassociation($document)
	union
	rl:removeOwnedMemberRepeatedByName(utc:stereotypesTOtypes($document)union utc:taggedValueDefinitionsTOtypes($document))
	union 
	wl:dataType("void", "public", "void_id")
	union(
		if($attributeAsComposition)then(utc:attributesTOassociation($document))
		else()
	)
};

declare function utc:actorTOclass($document as node(), $attributeAsComposition as xs:boolean) as node()*{
		for $actor in rl:getActors($document)
		return(
				wl:class($actor, rl:getGeneralizationFromOM($actor), utc:getStereotypeTOGeneralitation($document, $actor), (), $attributeAsComposition, utc:getTagsTOAttribute($document, $actor))
		)
};

declare function utc:useCaseTOclass($document as node(), $attributeAsComposition as xs:boolean) as node()*{
		for $useCase in rl:getUseCases($document)
		return(
			 if((rl:countStereotypesFromOM($useCase)=0) or (utc:isUseCaseInherited($useCase))=true())then(
			 	wl:class($useCase, rl:getGeneralizationFromOM($useCase), utc:getStereotypeTOGeneralitation($document, $useCase), utc:getIncludesTOAttribute($document, $useCase), $attributeAsComposition, utc:getTagsTOAttribute($document, $useCase))
			 )
			 else(
			 )
		)
};

declare function utc:isUseCaseInherited($useCase as node()) as xs:boolean{
		if(fn:count(
			let $taggedValues := rl:getTaggedValuesFromOM($useCase)
			for $tag in $taggedValues
			return(
					if((fn:compare($tag/@value,"Generalization")=0) and (fn:compare($tag/@tag,"Relation")=0))then(
						 $tag
					)
					else()
			)
		)>0)then(1 cast as xs:boolean)
		else (0 cast as xs:boolean)
};


declare function utc:includesTOassociation($document as node()) as node()*{
	for $om in $document//ownedMember, $inc in $om/include
		return(
			 if((rl:countStereotypesFromOM(rl:getOwnedMemberByID($document, $inc/@addition))=0) or (utc:isUseCaseInherited(rl:getOwnedMemberByID($document, $inc/@addition))))then(
			 		wl:association((), $om, $inc, $inc/@xmi:id, rl:getOwnedMemberByID($document, $inc/@addition)/@name, fn:false(), fn:false(), fn:false(), fn:true(), fn:false(), ())
			 )
			 else(
			 )
		)
};

declare function utc:attributesTOassociation($document as node()) as node()*{
	for $om in $document//ownedMember, $inc in $om/include
	return(
		for $st in rl:getStereotypeTaggedAsComposition($document, rl:getOwnedMemberByID($document, $inc/@addition))
		return( 
				 wl:association((), (), $st, $om/@xmi:id, rl:getOwnedMemberByID($document, $inc/@addition)/@name, fn:false(), fn:false(), fn:false(), fn:true(), fn:true(), fn:substring($inc/@addition, 2, 3))
		)
	)
};

declare function utc:associationUCDTOassociationCD($document as node()) as node()*{
	for $om in rl:getAssociations($document)
	let $ownedEnds := rl:getOwnedEnds($om)
		return(
			wl:association($om, $ownedEnds[1], $ownedEnds[2], $om/@xmi:id, rl:getOwnedMemberByID($document, $ownedEnds[2]/@type)/@name, false(), false(), false(), fn:false(), fn:false(), ())
		)
};

declare function utc:taggedValueDefinitionsTOtypes($document as node()) as node()*{
	for $st in rl:getStereotypes($document)
	where (fn:compare($st/@xmi:id, "Include_Include_id")!=0)
	return(
		for $tag in rl:getTagsFromStereotype($st)
		where (fn:compare($tag/@name, "Relation")!=0)
		return(
			wl:dataType($tag)
		)
	)
};

declare function utc:stereotypesTOtypes($document as node()) as node()*{
	for $st in rl:getStereotypes($document)
	return(
		if((fn:compare($st/@xmi:id, "Include_Include_id")!=0)and(fn:compare($st/@xmi:id, "Requirement_requirement_id")!=0)and(fn:compare($st/@xmi:id, "UseCase_UseCase_id")!=0)and(fn:compare($st/@xmi:id, "Class_Actor_id")!=0)and(fn:compare($st/@xmi:id, "Class_dataType_id")!=0))then(
			wl:dataType($st)
		)
		else(
		)
	)
};

declare function utc:getIncludesTOAttribute($document as node(), $ownedMember as node()) as node()*{
	for $inc in $ownedMember/include
	let $included:=rl:getOwnedMemberByID($document, $inc/@addition)
	let $stereotypes:=rl:getStereotypesFromOM($included)
	return(
		if(utc:isUseCaseInherited($included)) then()
		else(
			if(fn:count($stereotypes)>0)then $included
			else()
		)
	)
};

declare function utc:getTagsTOAttribute($document as node(), $ownedMember as node()) as node()*{
	for $in in $ownedMember/include
	let $included:=rl:getOwnedMemberByID($document, $in/@addition)
	let $tags:=rl:getTaggedValuesFromOM($included, "ComplementaryClass")
	return(
		if(fn:count($tags)>0)then $included
		else()
	)
};

declare function utc:getStereotypeTOGeneralitation($document as node(), $ownedMember as node()) as node()*{
		let $taggedValues := rl:getTaggedValuesFromOM($ownedMember)
		for $tag in $taggedValues
		where $tag/@value="Generalization" and $tag/@tag = "Relation"
		return(
				$ownedMember//appliedStereotype[@xmi:value=(rl:getStereotypeByName($document, $tag//stereotype/@xmi:value))/@xmi:id]
		)
};

declare function utc:useCaseANDactivitiesTOclass($document as node(), $attributeAsComposition as xs:boolean) as node()*{
		for $useCase in rl:getUseCases($document)
		return(
			 if((rl:countStereotypesFromOM($useCase)=0) or (utc:isUseCaseInherited($useCase))=true())then(
			 	wl:class($useCase, rl:getGeneralizationFromOM($useCase), utc:getStereotypeTOGeneralitation($document, $useCase), utc:getIncludesTOAttribute($document, $useCase), utc:getActivitiesTOAttribute($useCase), utc:getEdgesTOAttribute($useCase), $attributeAsComposition, utc:getTagsTOAttribute($document, $useCase) union utc:getTagsFromActivitiesTOAttribute($useCase) union utc:getTagsFromEdgesOfActivitiesTOAttribute($useCase))
			 )
			 else(
			 )
		)
};

declare function utc:activityTOAssociation($document as node(), $useCase as node(), $activity as node()) as node()*{
		for $st in rl:getStereotypeTaggedAsComposition($document, $activity)
		return(
				wl:association((), (), $st, $useCase/@xmi:id, fn:replace($activity/@name, "^.*?]", ""), fn:false(), fn:false(), fn:false(), fn:true(), fn:true(), fn:substring($activity/@xmi:id, 2, 3))
		)
};

declare function utc:getActivitiesTOAttribute($useCase as node()) as node()*{
	for $act in rl:getActivitiesFromUseCase($useCase)
	return
		if(count(rl:getStereotypesFromOM($act))>0) then $act
		else()
};

declare function utc:getEdgesOfActivitiesTOAttribute($activity as node()) as node()*{
	for $edge in rl:getEdgesFromActivity($activity)
	return
		if(count(rl:getStereotypesFromOM($edge))>0) then $edge 
		else()
};

declare function utc:getTagsFromActivitiesTOAttribute($useCase as node()) as node()*{
	for $act in utc:getActivitiesTOAttribute($useCase)
	let $tags:=rl:getTaggedValuesFromOM($act, "ComplementaryClass")
	return(
		if(fn:count($tags)>0)then $act
		else()
	)
};

declare function utc:getTagsFromEdgesOfActivitiesTOAttribute($useCase as node()) as node()*{
	for $act in rl:getActivitiesFromUseCase($useCase), $edge in $act
	let $tags:=rl:getTaggedValuesFromOM($edge, "ComplementaryClass")
	return(
		if(fn:count($tags)>0)then $edge
		else()
	)
};

declare function utc:repeatedEdgeOfActivity($activities as node()*, $edgeToCompare as node()) as node()*{
		for $a in $activities
		return(
				for $edge in utc:getEdgesOfActivitiesTOAttribute($a)
				where $edge/@name=$edgeToCompare/@name
				return(
					$edge
				)
		)
};

declare function utc:isFirstOccurrenceOfEdge($activities as node()*, $edgeToCompare as node()) as xs:boolean{
	if((
		let $edges:=utc:repeatedEdgeOfActivity($activities, $edgeToCompare)
		return(
			rl:indexOfNode($edges, $edgeToCompare)
		)
	)=1)then(
		fn:true()
	)else(
		fn:false()
	)
};

declare function utc:activitiesTOassociation($document as node()) as node()*{
	for $useCase in rl:getUseCases($document)
	return(
		for $act in utc:getActivitiesTOAttribute($useCase)
		return(
			for $st in rl:getStereotypeTaggedAsComposition($document, $act)
			return(
				wl:association((), (), $st, $useCase/@xmi:id, fn:replace($act/@name, "^.*?]", ""), fn:false(), fn:false(), fn:false(), fn:true(), fn:true(), fn:substring($act/@xmi:id, 2, 3))
			)
		)
	)
};

declare function utc:edgesTOassociation($document as node()) as node()*{
	for $useCase in rl:getUseCases($document)
	let $activities:= rl:getActivitiesFromUseCase($useCase)
	return(
		for $act in $activities
		return(
			for $edge in utc:getEdgesOfActivitiesTOAttribute($act)
			where utc:isFirstOccurrenceOfEdge($activities, $edge)
			return(
				for $st in rl:getStereotypeTaggedAsComposition($document, $edge)
				return(
					wl:association((), (), $st, $useCase/@xmi:id, fn:replace($edge/@name, "^.*?]", ""), fn:false(), fn:false(), fn:false(), fn:true(), fn:true(), fn:substring($edge/@xmi:id, 2, 3))
				)
			)
		)
	)
}; 

declare function utc:getEdgesTOAttribute($useCase as node()) as node()*{
		let $activities:= rl:getActivitiesFromUseCase($useCase)
		for $act in $activities
		return(
			for $edge in utc:getEdgesOfActivitiesTOAttribute($act)
			where utc:isFirstOccurrenceOfEdge($activities, $edge)
			return(
				$edge
			)
		)
};