module namespace rl="XMI Read Library";

declare namespace xmi="http://schema.omg.org/spec/XMI/2.1";
declare namespace uml="http://schema.omg.org/spec/UML/2.0";
declare namespace fn="http://www.w3.org/2005/xpath-functions";
declare namespace xs="http://www.w3.org/2001/XMLSchema";

declare function rl:getOwnedMember($document as node(), $id as xs:string, $isAbstract as xs:boolean, $type as xs:string) as node()*{
		for $om in $document//ownedMember
		where (($om/@xmi:id=$id) and ($om/@isAbstract=$isAbstract) and ($om/@xmi:type=$type))
		return $om
};

declare function rl:getOwnedMemberByType($document as node(), $type as xs:string) as node()*{
		for $om in $document//ownedMember
		where ($om/@xmi:type=$type)
		return $om
};

declare function rl:getOwnedMemberByID($document as node(), $id as xs:string) as node(){
		for $om in $document//ownedMember
		where ($om/@xmi:id=$id)
		return $om
};

declare function rl:getPackages($x as node()) as node()*{
	for $b in $x//ownedMember
	where $b/@xmi:type="uml:Package"
	return $b
};

declare function rl:getPackageByClassID($id as xs:string, $packages as node()*) as node()*{
	for $package in $packages
	return(
		let $auxiliar:=$package
		for $b in $package//ownedMember
		where $b/@xmi:type="uml:Class" and $b/@xmi:id=$id
		return $auxiliar		
	)
};

declare function rl:getClasses($document as node()) as node()*{
	for $om in $document//ownedMember
	where $om/@xmi:type="uml:Class"
	return $om
};

declare function rl:getClassGeneralizations($x as node()) as node()*{
	for $b in $x/generalization
	return $b
};

declare function rl:getClassConstructors($class as node()) as node()*{
	for $oo in $class/ownedOperation
	where $oo/@name = $class/@name
	return $oo
};

declare function rl:getClassNonConstructorMethods($class as node()) as node()*{
	for $oo in $class/ownedOperation
	where $oo/@name != $class/@name
	return $oo
};

declare function rl:getMethodReturnParameter($ownedOperation as node()) as node()*{
	for $op in $ownedOperation/ownedParameter
	where $op/@kind = "return"
	return $op
};

declare function rl:getMethodInOutParameters($ownedOperation as node()) as node()*{
	for $op in $ownedOperation/ownedParameter
	where $op/@kind != "return"
	return $op
};

declare function rl:getClassAttributes($class as node()) as node()*{
	for $b in $class/ownedAttribute
	where $b/@xmi:type="uml:Property"
	return $b
};

declare function rl:getAttributeByName($class as node(), $name as node()) as node()*{
	for $att in rl:getAttributesFromClass($class)
	where $att/@name=$name
	return $att
};

declare function rl:getAttributesNames($class as node()) as node()*{
	for  $attribute in $class//ownedAttribute
	return $attribute/@name
};

declare function rl:getAttributesFromClass($class as node()) as node()*{
	for  $attribute in $class//ownedAttribute
	return $attribute
};

declare function rl:getClassCompositions($x as node(), $assoc as node()*) as node()*{
	for $b in $assoc
	where (rl:getCompositionDest($b)/@xmi:type = "uml:Property") and 
				(rl:getCompositionDest($b)/@type = $x/@xmi:id)
	return $b
};

declare function rl:getCompositionSource($assoc as node()) as node()*{
	for $b in rl:getAssociationEnds($assoc)
	where($b/@aggregation = "composite")
	return $b
};

declare function rl:getCompositionDest($assoc as node()) as node()*{
	for $b in rl:getAssociationEnds($assoc)
	where($b/@aggregation = "none")
	return $b
};

declare function rl:getAssociationEnds($assoc as node()) as node()*{
	for $b in $assoc
	return $b/ownedEnd
};

declare function rl:getStereotypeTaggedAsComposition($document as node(), $ownedMember as node()) as node()*{
	for $st in $ownedMember/xmi:Extension/appliedStereotype 
	return(
		if(
		(
		fn:count
		(
			for $tag in rl:getTagsByStereotypeName($ownedMember, (rl:getStereotypeByID($document, $st/@xmi:value))/@name)
			where $tag/@value="Composition"
			return $tag
		)
		)>0
		)then($st)
		else()
	)
};

declare function rl:getTagsByValue($tags as node()*, $value as xs:string) as node()*{
	for $t in $tags
	where $t/@value=$value
	return($t)
};

declare function rl:getStereotypeByName($document as node(), $name as xs:string) as node()*{
		for $st in $document//ownedMember[@xmi:type="uml:Stereotype"]
		where ($st/@name=$name)
		return $st
};

declare function rl:getTagsByStereotypeName($x as node(), $name as xs:string) as node()*{
		for $b in $x//taggedValue
		where (fn:compare($b//stereotype/@xmi:value, $name)=0)
		return $b
};

declare function rl:getStereotypeByID($document as node(), $id as xs:string) as node(){
		for $st in $document//ownedMember[@xmi:type="uml:Stereotype"]
		where ($st/@xmi:id=$id)
		return $st
};

declare function rl:getOMNameByID($doc as node(), $id as xs:string) as xs:string{
		for $x in $doc//ownedMember
		where ($x/@xmi:id=$id)
		return $x/@name
};

declare function rl:getModelName($document as node()) as xs:string{
		for $mod in $document//uml:Model
		return $mod/@name
};

declare function rl:getModelID($document as node()) as xs:string{
		for $mod in $document//uml:Model
		return $mod/@xmi:id
};

declare function rl:getActors($document as node()) as node()*{
	for $om in $document//ownedMember
	where $om/@xmi:type="uml:Actor"
	return $om
};

declare function rl:getStereotypes($document as node()) as node()*{
	for $om in $document//ownedMember
	where $om/@xmi:type="uml:Stereotype"
	return $om
};

declare function rl:getIncludes($document as node()) as node()*{
	for $in in $document//ownedMember//include
	return $in
};

declare function rl:getIncludesByAddition($document as node(), $addition as xs:string) as node()*{
	for $in in $document//ownedMember//include
	where $in/@addition=$addition
	return $in
};

declare function rl:getUseCases($document as node()) as node()*{
	for $om in $document//ownedMember
	where $om/@xmi:type="uml:UseCase"
	return $om
};

declare function rl:getUseCase($document as node(), $name as xs:string) as node(){
	for $om in $document//ownedMember
	where $om/@xmi:type="uml:UseCase" and $om/@name=$name
	return $om
};

declare function rl:getAbsTractClasses($document as node()) as node()*{
	for $cla in $document//ownedMember[@xmi:type="uml:Class"]
	where $cla//@isAbstract="true"
	return $cla
};


declare function rl:getProperties($document as node()) as node()*{
	for $pr in $document//ownedMember
	where $pr/@xmi:type="uml:Property"
	return $pr
};

declare function rl:getOperations($document as node()) as node()*{
	for $op in $document//ownedMember
	where $op/@xmi:type="uml:Operation"
	return $op
};

declare function rl:getAssociations($document as node()) as node()*{
	for $oa in $document//ownedMember
	where $oa/@xmi:type="uml:Association"
	return $oa
};

declare function rl:getOwnedEnds($x as node()) as node()*{
	for $b in $x/ownedEnd
	return $b
};

declare function rl:getGeneralizationFromOM($ownedMember as node()) as node()*{
	for $gen in $ownedMember/generalization
	return $gen
};

declare function rl:getStereotypesFromOM($ownedMember as node()) as node()*{
	for $st in $ownedMember/xmi:Extension/appliedStereotype
	where $st/@xmi:value!="UseCase_UseCase_id"
	return $st
};

declare function rl:getTaggedValuesFromOM($ownedMember as node()) as node()*{
	for $tag in $ownedMember/xmi:Extension/taggedValue
	return $tag
};

declare function rl:getTaggedValuesFromOM($ownedMember as node(), $name as xs:string) as node()*{
	for $tag in $ownedMember/xmi:Extension/taggedValue
	where (fn:contains($tag/@tag, $name))
	return $tag
};

declare function rl:getTagsFromStereotype($x as node()) as node()*{
	for $b in $x//taggedValueDefinitions/taggedValueDefinition
	return $b
};

declare function rl:countStereotypesFromOM($x as node()) as xs:integer{
	let $count:=fn:count(rl:getStereotypesFromOM($x))
	return $count
};

declare function rl:getActivitiesFromUseCase($useCase as node()) as node()*{
	(for $act in $useCase//ownedMember
	where $act/@xmi:type="uml:Activity"
	return $act)
	union
	(for $act in $useCase//node
	where $act/@xmi:type="uml:Activity"
	return $act)
};

declare function rl:getEdgesFromActivity($activity as node()) as node()*{
	for $edge in $activity/edge
	where $edge/@xmi:type="uml:ControlFlow"
	return $edge
};

declare function rl:indexOfNode($nodes as node()*, $nodeToFind as node())  as xs:integer*{  
  for $seq in (1 to fn:count($nodes))
  return $seq[$nodes[$seq] is $nodeToFind]
};

declare function rl:repeatedNodesOfList($listOfNodes as node()*, $nodeToCompare as node()) as node()*{
		for $element in $listOfNodes
		where (($element/@name=$nodeToCompare/@name) and ($element/@xmi:type=$nodeToCompare/@xmi:type))
		return(
			$element
		)
};

declare function rl:isFirstOccurrenceOfNode($listOfNodes as node()*, $nodeToCompare as node()) as xs:boolean{
	if((
		let $nodes:=rl:repeatedNodesOfList($listOfNodes, $nodeToCompare)
		return(
			rl:indexOfNode($nodes, $nodeToCompare)
		)
	)=1)then(
		fn:true()
	)else(
		fn:false()
	)
};

declare function rl:removeOwnedMemberRepeatedByName($list as node()*){
	for $x in distinct-values($list//ownedMember/@name)
	return rl:getElementFromListByName($x, $list//ownedMember)
};

declare function rl:getElementFromListByName($name as xs:string, $list as node()*) as node(){
	(for $x in $list
	where (fn:compare($x/@name, $name)=0)
	return $x)[1]
};