/*****************************************************************************
 * This file is part of the Prolog Development Tools (ProDT)
 *
 * Author: Claudio Cancinos
 * WWW: https://sourceforge.net/projects/prodevtools
 * Copyright (C): 2008, Claudio Cancinos
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; If not, see <http://www.gnu.org/licenses/>
 ****************************************************************************/
package ar.com.tadp.prolog.core.compiler.errorparsers;

import java.util.StringTokenizer;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarker;

import ar.com.tadp.prolog.core.PrologCorePlugin;
import ar.com.tadp.prolog.core.builder.ResourceUtil;
import ar.com.tadp.prolog.core.compiler.CompilerOutput;
import ar.com.tadp.prolog.core.compiler.CompilerOutputItem;

/**
 * @author ccancino
 *
 */
public class XSBCompilerErrorsParser extends CompilerErrorsParser {
	private final static Pattern SYNTAX_ERROR = Pattern.compile("^\\*+ syntax error$"); 
	private final static Pattern WARNING = Pattern.compile("^\\*+ warning \\*+(.+):\\((.+),(.+)-(.+)\\)::(.+)$");

	@Override
	protected void doParse(IFile file, String outputString, CompilerOutput compilerOutput) {
		StringTokenizer tn = new StringTokenizer(outputString, ResourceUtil.LINE_SEPARATOR);
        while(tn.hasMoreTokens()) {
        	String line = tn.nextToken();
        	
			try {
				Matcher m = SYNTAX_ERROR.matcher(line.trim());
				if(m.matches()){
					this.addError(compilerOutput, tn.nextToken(), tn.nextToken(), file);
				}
				
				m = WARNING.matcher(line);
				if(m.matches()){
					this.addWarning(compilerOutput, m);
				}
			} catch (Exception e) {
				PrologCorePlugin.log("Error trying to parse error line: " + line, e);
			}
		}
	}

	private void addWarning(CompilerOutput compilerOutput, Matcher m) {
		// TODO Auto-generated method stub
		
	}

	private void addError(CompilerOutput compilerOutput, String beforeError, String afterError, IFile file) {
		String before = beforeError.substring(beforeError.indexOf(">>>") + 3, beforeError.indexOf("<--- HERE?")).trim();
		String after = afterError.substring(afterError.indexOf(">>>") + 3).trim();
		int errorLine = this.getLine(file, before + after);
		errorLine = (errorLine == -1) ? this.getLine(file, before) : errorLine;
		
		CompilerOutputItem item = new CompilerOutputItem(IMarker.SEVERITY_ERROR);
		item.setFileName(file.getLocation().toPortableString());
		item.setLine(errorLine);
		item.setStartColumn(this.getOffset(item.getLine()));
		item.setEndColumn(this.getLength(item.getLine()));
		item.setComment(beforeError + "\n" + afterError);
		compilerOutput.addError(item);
	}

}
