/*****************************************************************************
 * This file is part of the Prolog Development Tools (ProDT)
 *
 * Author: Claudio Cancinos
 * WWW: https://sourceforge.net/projects/prodevtools
 * Copyright (C): 2008, Claudio Cancinos
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; If not, see <http://www.gnu.org/licenses/>
 ****************************************************************************/
package ar.com.tadp.prolog.core.console.ui;

import java.util.Iterator;
import java.util.List;

import org.eclipse.debug.core.ILaunch;
import org.eclipse.dltk.console.IScriptInterpreter;
import org.eclipse.dltk.console.ui.IScriptConsole;
import org.eclipse.dltk.console.ui.IScriptConsoleFactory;
import org.eclipse.dltk.console.ui.ScriptConsole;
import org.eclipse.dltk.console.ui.ScriptConsoleFactoryBase;
import org.eclipse.dltk.core.IScriptProject;


public class PrologConsoleFactory extends ScriptConsoleFactoryBase implements IScriptConsoleFactory {

	@Override
	protected ScriptConsole createConsoleInstance() {
		return this.createConsole(this.createPrologInterpreter(null), "Stand Alone");
	}

	private ScriptConsole createConsole(PrologScriptInterpreter interpreter, String consoleName) {
		PrologScriptConsole console = new PrologScriptConsole(interpreter, consoleName);
		interpreter.setConsole(console);
		//HACK this is just to display the "welcome message" of the underlying interpreter.
		console.executeCommand("\n");
		return console;
	}

	public IScriptConsole openConsole(IScriptInterpreter interpreter, String id,ILaunch launch) {
		ScriptConsole console = new PrologScriptConsole(interpreter, id);
		registerAndOpenConsole(console);
		return console;
	}

	public void openConsole(List files, IScriptProject project) {
		PrologScriptInterpreter interpreter = this.createPrologInterpreter(project);
		PrologScriptConsole console = (PrologScriptConsole) this.createConsole(interpreter, files.toString());
		if (console != null) {
			registerAndOpenConsole(console);
			for (Iterator iterator = files.iterator(); iterator.hasNext();) {
				String path = (String) iterator.next();
				console.executeCommand("consult(\'" + path + "\').");
			}
		}
	}

	private PrologScriptInterpreter createPrologInterpreter(IScriptProject project) {
		PrologScriptInterpreter interpreter = null;
		try {
			interpreter = (project != null) ? new PrologScriptInterpreter(project) : new PrologScriptInterpreter();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return interpreter;
	}

}
