/*****************************************************************************
 * This file is part of the Prolog Development Tools (ProDT)
 *
 * Author: Claudio Cancinos
 * WWW: https://sourceforge.net/projects/prodevtools
 * Copyright (C): 2008, Claudio Cancinos
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; If not, see <http://www.gnu.org/licenses/>
 ****************************************************************************/
package ar.com.tadp.prolog.core.interpreter;

import java.io.IOException;
import java.util.regex.Matcher;

import org.eclipse.core.resources.IFile;
import org.eclipse.dltk.core.IScriptProject;

import ar.com.tadp.prolog.core.PrologCorePlugin;
import ar.com.tadp.prolog.core.builder.ResourceUtil;
import ar.com.tadp.prolog.core.commandline.CommandResponse;
import ar.com.tadp.prolog.core.commandline.SyncCommandLine;
import ar.com.tadp.prolog.core.compiler.CompilerOutput;
import ar.com.tadp.prolog.core.compiler.errorparsers.XSBCompilerErrorsParser;

public class XSBPrologInstallType extends AbstractPrologInstallType {
	private static final String[] INTERPRETER_NAMES = { "xsb", "xsb.bat" };
	private XSBCompilerErrorsParser errorsParser = new XSBCompilerErrorsParser();
	
	public String getName() {
		return "XSB Prolog";
	}

	protected String[] getPossibleInterpreterNames() {
		return INTERPRETER_NAMES;
	}

	public Process createProcess(String command, String[] envp) {
		try {
			if(command.endsWith(".bat")) {
				String[] args = new String[]{ "cmd", "/c", command};
				return Runtime.getRuntime().exec(args);
			} else {
				String[] args = new String[]{ command, "-l"};
				return Runtime.getRuntime().exec(args);
			}
		} catch (IOException e) {
			PrologCorePlugin.log("Cannot execute Prolog Interpreter.", e);
		}
		return null;
	}

	@Override
	public CompilerOutput compile(IFile sourceFile, String outputFile, IScriptProject project, PrologInstall prologInstall) {
		SyncCommandLine interpreter = null;
		try {
			PrologInstall install = (PrologInstall) this.getInterpreterInstalls()[0];
			interpreter = install.executeSynchronousInterpreter();
			String command = "compile('" + sourceFile.getLocation().toPortableString() + "','" + outputFile.replaceAll(Matcher.quoteReplacement("\\"), "/") + "').";
			CommandResponse response = interpreter.executeCommand(command);

			return this.parseOutput(sourceFile, response.getError() + ResourceUtil.LINE_SEPARATOR + response.getOutput());
		} catch (IOException e) {
		} finally {
			if(interpreter != null) {
				interpreter.kill();
			}
		}
		return new CompilerOutput();
	}
	
	@Override
	protected CompilerOutput parseOutput(IFile sourceFile, String result) {
		super.parseOutput(sourceFile, result);
		return this.errorsParser.parse(result, sourceFile);
	}
	
}
